#!/usr/bin/env python
"""

    Copyright 2011 Vince Spicer <vinces1979@gmail.com>

    As long as you retain this notice you can do whatever you want with this
    stuff. If we meet some day, and you think this stuff is worth it,
    you can buy me a beer in return.

    2015 - Luar was here <cyberplant@gmail.com>

"""

import string
import re

from random import SystemRandom
choice = SystemRandom().choice
randint = SystemRandom().randint

LowercaseLetters = string.ascii_lowercase
UpperCase = string.ascii_uppercase
Digits = string.digits
Symbols = string.punctuation

HasCaps = re.compile("[A-Z]")
HasNumerals = re.compile("[0-9]")
HasSymbols = re.compile(r"[%s]" % re.escape(Symbols))
HasAmbiguous = re.compile("[B8G6I1l|0OQDS5Z2]")


def replaceRandomChar(letter, word, pos=None):
    if not pos:
        pos = randint(0, len(word)-1)
    word = list(word)
    word[pos] = letter
    return "".join(word)


def pwgen(pw_length=20, num_pw=1, no_numerals=False, no_capitalize=False,
          capitalize=False, numerals=False, no_symbols=True, symbols=False,
          allowed_symbols=None, no_ambiguous=False):
    """Generate a random password.

    @param pw_length: The length of the password to generate [default: 20]
    @param num_pw: The number of passwords to generate [default: 1]
    @param no_numerals: Don't include numbers in the passwords [default: False]
    @param numerals: Enforce at least one number to be in the
                     password [default: False]
    @param no_capitalize: Don't include capital letters in the
                          password [default: False]
    @param capitalize: Enforce at least one capital letter to be in
                       the password [default: False]
    @param no_symbols: Don't include symbols in the password [default: True]
    @param symbols: Enforce at least one symbol to be in the
                    password [default: False]
    @param allowed_symbols: a string containing allowed
                            symbols [default: string.punctuation]
    @param no_ambigous: Don't include ambigous characters [default: False]

    """

    global Symbols, HasSymbols
    letters = LowercaseLetters
    if not no_capitalize:
        letters += UpperCase
    if not no_numerals:
        letters += Digits
    if not no_symbols:
        if allowed_symbols is not None:
            Symbols = allowed_symbols
            HasSymbols = re.compile(r"[%s]" % re.escape(Symbols))
        letters += Symbols

    passwds = []
    while len(passwds) < int(num_pw):
        passwd = "".join(choice(letters) for x in range(pw_length))
        if capitalize and not HasCaps.search(passwd):
            passwd = replaceRandomChar(choice(UpperCase), passwd)
        if numerals and not HasNumerals.search(passwd):
            passwd = replaceRandomChar(choice(Digits), passwd)
        if symbols and not HasSymbols.search(passwd):
            passwd = replaceRandomChar(choice(Symbols), passwd)
        if no_ambiguous and HasAmbiguous.search(passwd):
            continue
        passwds.append(passwd)

    if len(passwds) == 1:
        return passwds[0]

    return passwds




import pwgen
import argparse

"""

    Contribution at Aug 2014 by Luar Roji <cyberplant@gmail.com>

    This module adds the command line interface to be able to run as the
    regular pwgen.

"""


def main(data=None):
    parser = argparse.ArgumentParser(description='random PassWord GENerator')

    parser.add_argument('pw_length', default='8', type=int,
                        nargs='?',
                        help="Password length")

    parser.add_argument('--capitalize', '-c', action='store_true',
                        dest="capitalize",
                        help="Include at least one capital letter in "
                             "the password")

    parser.add_argument('--no-capitalize', '-A', action='store_true',
                        dest="no_capitalize",
                        help="Don't include capital letters in the password")

    parser.add_argument('--numerals', '-n', action='store_true',
                        dest='numerals',
                        help="Include at least one number in the password")
    parser.add_argument('--no-numerals', '-0', action='store_true',
                        dest='no_numerals',
                        help="Don't include numbers in the password")

    parser.add_argument('--symbols', '-y', action='store_true', dest='symbols',
                        help="Include at least one special symbol in "
                             "the password")

    # parser.add_argument('--secure', '-s', action='store_true', dest='secure',
    #                     help="Generate completely random passwords")

    parser.add_argument('--ambiguous', '-B', action='store_true',
                        dest='no_ambiguous',
                        help="Don't include ambiguous characters in "
                             "the password")

    # parser.add_argument('-H or --sha1=path/to/file[#seed]
    #                     help="Use sha1 hash of given file as a (not so) "
    #                          "random generator")

    # parser.add_argument('-C', action='store_true', dest='print_in_columns',
    #                     help="Print the generated passwords in columns")

    parser.add_argument('-1', action='store_const', const=1, default='160',
                        dest='num_pw',
                        help="Don't print the generated passwords in columns")

    # parser.add_argument('--no-vowels', '-v', action='store_false',
    #                     dest='vowels',
    #                     help="Do not use any vowels so as to avoid
    #                          "accidental nasty words")

    args = parser.parse_args()

    d = vars(args)

    result = pwgen.pwgen(**d)

    if type(result) == list:
        for y in range(20):
            print " ".join([result.pop() for x in range(8)])
    else:
        print result
